# -*- coding: utf-8 -*-
import logging
from datetime import date
from dateutil.relativedelta import relativedelta

from odoo import api, models, fields, SUPERUSER_ID, _
from odoo.exceptions import UserError
from odoo.tools import format_amount

_logger = logging.getLogger(__name__)


class SaleOrderLine(models.Model):
    _inherit = 'sale.order.line'

    supplier_id = fields.Many2one('product.supplierinfo', compute='update_supplier', store=True, string="Product Supplier")

    @api.depends('product_id')
    def update_supplier(self):
        for line in self:
            if line.product_id:
                line.supplier_id = line.product_id.seller_ids and line.product_id.seller_ids[-1]

    def _purchase_service_match_supplier(self, warning=True):
        supplier = self.supplier_id or super()._purchase_service_match_supplier(warning=False)
        if not supplier and warning:
            raise UserError(_("There is no vendor on the line associated to the product %r.") % (self.product_id))
        return supplier

    def _purchase_service_match_purchase_order(self, partner, company=False):
        super_method = super()._purchase_service_match_purchase_order
        res = super_method(partner.commercial_partner_id, company=self.env.ref('base.main_company'))
        if not res:
            res = super_method(partner, company=self.env.ref('base.main_company'))

        currency_id = partner.property_purchase_currency_id or self.env.ref('base.main_company').currency_id
        res = res.filtered(lambda po: po.currency_id == currency_id and po.purchase_type == 'apps')

        # raising an error here would prevent sales from proceeding seamlessly until these POs are merged
        # if several POs are found, return the oldest one; they will probably be merged later anyway
        return res[-1:]

    def _purchase_service_prepare_order_values(self, supplierinfo):
        """ Returns the values to create the purchase order from the current SO line.
         :param supplierinfo: record of product.supplierinfo
         :rtype: dict
        """
        res = super()._purchase_service_prepare_order_values(supplierinfo)
        partner_supplier = supplierinfo.partner_id.commercial_partner_id
        if res['partner_id'] != partner_supplier.id:
            fpos = self.env['account.fiscal.position'].sudo()._get_fiscal_position(partner_supplier)
            res['partner_id'] = partner_supplier.id
            res['partner_ref'] = partner_supplier.ref
            res['currency_id'] = partner_supplier.property_purchase_currency_id.id or self.env.company.currency_id.id
            res['payment_term_id'] = partner_supplier.property_supplier_payment_term_id.id
            res['fiscal_position_id'] = fpos.id
        res['purchase_type'] = 'apps'
        res['user_id'] = SUPERUSER_ID  # avoid the first buyer to be the user_id of the PO
        return res

    def _purchase_get_date_order(self, supplierinfo):
        today = date.today()
        next_po_order_date = int(self.env['ir.config_parameter'].sudo().get_param('service_purchase.next_po_order_date', '20'))
        date_order = today.replace(day=next_po_order_date)
        if today.day >= next_po_order_date:
            date_order += relativedelta(months=1)
        return date_order

    def _purchase_service_prepare_line_values(self, purchase_order, quantity=False):
        res = super()._purchase_service_prepare_line_values(purchase_order, quantity=quantity)

        if purchase_order.purchase_type == 'apps':
            if self.supplier_id:  # don't use cheapest price, but use the one stored on add to cart
                price_unit, taxes = self._purchase_service_get_price_unit_and_taxes(self.supplier_id, purchase_order)
                res['price_unit'] = price_unit
                res['taxes_id'] = [(6, 0, taxes.ids)]

            res['name'] = "Commission for %r on %r - %s" % (
                self.product_id.name,
                self.order_id.name,
                format_amount(self.env, self.price_subtotal, self.currency_id)
            )

        if quantity < 0:
            res['name'] = _("Refund of %r") % res['name']
        return res

    def _purchase_service_get_company(self):
        return self.env.ref('base.main_company')

    def _create_or_update_po_lines(self, subtract=False, qty=0):
        """
        This function is called when an invoice containing a to-purchase product is paid.
        It will create a purchase_order for the vendor if there is no existing draft PO
        for this vendor. Otherwise, the existing draft PO will be updated by adding the
        corresponding purchase order lines.
        """
        self.ensure_one()
        # only create a commission line if another doesn't already exist:
        # - with a positive quantity if we are not subtracting
        # - with a negative quantity otherwise
        # without sudo(), purchase_line_ids is scoped to current company's lines, but the PO's company might be different
        if not self.sudo().purchase_line_ids.filtered(lambda pl: pl.product_qty < 0 if subtract else pl.product_qty > 0):
            self.sudo()._purchase_service_create(quantity=qty * (-1 if subtract else 1))

    def _purchase_service_generation(self):
        # ignore computation of PO on SO confirmation / modification
        # It is deferred to the payment of the invoice.
        return
